/*
 * Copyright (c) 2001 Massachusetts Institute of Technology.
 *
 * This file is part of bgptools. 
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `COPYING' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file AUTHORS which should have accompanied this software 
 * for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the AUTHORS file.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


/*
 *     Nick Feamster (feamster@lcs.mit.edu)
 *     MIT Lab for Computer Science
 *     November 2001
 */


#ifndef __PREFIX_TREE_H__
#define __PREFIX_TREE_H__

#include <sys/types.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <fcntl.h>
#include <assert.h>
#include <string.h>

#include "bgp_dump.h"
#include "config.h"

#define DOT_DIR "/usr/local/bin"

typedef struct prefix_node {
  u_int32_t prefix;
  char depth;
  prefix_node *parent;
  prefix_node *left;
  prefix_node *right;
} prefix_node;



class prefix_tree {

  int node_num;
  prefix_node *root;
  int find_depth(u_int32_t prefix);

 public:
  prefix_tree();
  ~prefix_tree();
  
  void insert(prefix_node *p, prefix_node *cur, int depth=0);  /* insert prefix node into tree */
  void insert(struct update *upd); /* take update struct and insert pfx */
  void insert(char *filename);    /* read MRT update file and insert
				     into tree */
  


  prefix_node* prefix_tree::new_node(u_int32_t prefix, int depth, int bit);
  void print(prefix_node *p);
  
  void print_dot(prefix_node *p, char *filename);
  void pdh(prefix_node *p, FILE *fp);
  void pdh(prefix_node *p, int fp);

  prefix_node *getRoot() {return root;}
  void free_tree(prefix_node *p);
};


#endif  /* __PREFIX_TREE_H__ */
